package ch.hepia.shop_online;


import javax.jms.Connection;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;

import org.apache.activemq.ActiveMQConnectionFactory;


public class TransactionApi {
    private static int ackMode = Session.AUTO_ACKNOWLEDGE;
    private static String clientQueueName = "client.messages";

    private boolean transacted = false;

    private Session session;
    private MessageProducer producer;

    static class TransactionBuilder {
        private String ipBroker;
        public TransactionBuilder(final String ipBroker) {
            this.ipBroker = ipBroker;
        }
        public TransactionApi withMessage(MessageListener messageListener) {
            return new TransactionApi(this.ipBroker, messageListener);
        }
    }

    private TransactionApi(final String ipBroker, MessageListener messageListener) {

        final String brokerUrl = "tcp://" + ipBroker;

        ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(brokerUrl);
        try {
            Connection connection = connectionFactory.createConnection();
            connection.start();

            session = connection.createSession(transacted, ackMode);

            Destination mainTopic = session.createTopic(clientQueueName);

            //Setup a message producer to send message to the queue the server is consuming from
            producer = session.createProducer(mainTopic);
            producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);

            MessageConsumer responseConsumer = session.createConsumer(mainTopic);

            responseConsumer.setMessageListener(messageListener);

        } catch (JMSException e) {
            e.printStackTrace();
        }
    }

    public static TransactionBuilder withLocation(String ipBroker) {
        return new TransactionBuilder(ipBroker);
    }

    public void send(String msg) throws JMSException {
        producer.send( session.createTextMessage(msg) );
    }

    public void send(java.io.Serializable o) throws JMSException {
        producer.send( session.createObjectMessage(o) );
    }

}
